@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-"""The main entry point. Invoke as `datus' or `python -m datus'.
+"""The main entry point. Invoke as `undatum' or `python -m undatum`.
 
 """
 import sys
@@ -11,10 +11,6 @@ def main():
         exit_status = cli()
     except KeyboardInterrupt:
         print("Ctrl-C pressed. Aborting")
-#        from httpie.status import ExitStatus
-#        exit_status = ExitStatus.ERROR_CTRL_C
-
-#    sys.exit(exit_status.value)
     sys.exit(0)
 
 
